@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,6 +20,8 @@
 */
 #include "../SDL_internal.h"
 
+#if SDL_HAVE_BLIT_0
+
 #include "SDL_video.h"
 #include "SDL_blit.h"
 
@@ -480,4 +482,6 @@ SDL_CalculateBlit0(SDL_Surface * surface)
     return (SDL_BlitFunc) NULL;
 }
 
+#endif /* SDL_HAVE_BLIT_0 */
+
 /* vi: set ts=4 sw=4 expandtab: */
