@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -34,7 +34,6 @@
 
 #include "SDL_hints.h"
 #include "SDL_video.h"
-#include "SDL_assert.h"
 #include "SDL_endian.h"
 #include "SDL_pixels_c.h"
 
@@ -54,6 +53,92 @@
 #define LCS_WINDOWS_COLOR_SPACE    0x57696E20
 #endif
 
+static int readRlePixels(SDL_Surface * surface, SDL_RWops * src, int isRle8)
+{
+    /*
+    | Sets the surface pixels from src.  A bmp image is upside down.
+    */
+    int pitch = surface->pitch;
+    int height = surface->h;
+    Uint8 *start = (Uint8 *)surface->pixels;
+    Uint8 *end = start + (height*pitch);
+    Uint8 *bits = end-pitch, *spot;
+    int ofs = 0;
+    Uint8 ch;
+    Uint8 needsPad;
+
+#define COPY_PIXEL(x)   spot = &bits[ofs++]; if(spot >= start && spot < end) *spot = (x)
+
+    for (;;) {
+        if (!SDL_RWread(src, &ch, 1, 1)) return 1;
+        /*
+        | encoded mode starts with a run length, and then a byte
+        | with two colour indexes to alternate between for the run
+        */
+        if (ch) {
+            Uint8 pixel;
+            if (!SDL_RWread(src, &pixel, 1, 1)) return 1;
+            if (isRle8) {                   /* 256-color bitmap, compressed */
+                do {
+                    COPY_PIXEL(pixel);
+                } while (--ch);
+            } else {                         /* 16-color bitmap, compressed */
+                Uint8 pixel0 = pixel >> 4;
+                Uint8 pixel1 = pixel & 0x0F;
+                for (;;) {
+                    COPY_PIXEL(pixel0); /* even count, high nibble */
+                    if (!--ch) break;
+                    COPY_PIXEL(pixel1); /* odd count, low nibble */
+                    if (!--ch) break;
+                }
+            }
+        } else {
+            /*
+            | A leading zero is an escape; it may signal the end of the bitmap,
+            | a cursor move, or some absolute data.
+            | zero tag may be absolute mode or an escape
+            */
+            if (!SDL_RWread(src, &ch, 1, 1)) return 1;
+            switch (ch) {
+            case 0:                         /* end of line */
+                ofs = 0;
+                bits -= pitch;               /* go to previous */
+                break;
+            case 1:                         /* end of bitmap */
+                return 0;                    /* success! */
+            case 2:                         /* delta */
+                if (!SDL_RWread(src, &ch, 1, 1)) return 1;
+                ofs += ch;
+                if (!SDL_RWread(src, &ch, 1, 1)) return 1;
+                bits -= (ch * pitch);
+                break;
+            default:                        /* no compression */
+                if (isRle8) {
+                    needsPad = (ch & 1);
+                    do {
+                        Uint8 pixel;
+                        if (!SDL_RWread(src, &pixel, 1, 1)) return 1;
+                        COPY_PIXEL(pixel);
+                    } while (--ch);
+                } else {
+                    needsPad = (((ch+1)>>1) & 1); /* (ch+1)>>1: bytes size */
+                    for (;;) {
+                        Uint8 pixel;
+                        if (!SDL_RWread(src, &pixel, 1, 1)) return 1;
+                        COPY_PIXEL(pixel >> 4);
+                        if (!--ch) break;
+                        COPY_PIXEL(pixel & 0x0F);
+                        if (!--ch) break;
+                    }
+                }
+                /* pad at even boundary */
+                if (needsPad && !SDL_RWread(src, &ch, 1, 1)) return 1;
+                break;
+            }
+        }
+    }
+}
+
 static void CorrectAlphaChannel(SDL_Surface *surface)
 {
     /* Check to see if there is any alpha channel data */
@@ -106,26 +191,23 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
 
     /* The Win32 BMP file header (14 bytes) */
     char magic[2];
-    /* Uint32 bfSize = 0; */
-    /* Uint16 bfReserved1 = 0; */
-    /* Uint16 bfReserved2 = 0; */
-    Uint32 bfOffBits = 0;
+    /* Uint32 bfSize; */
+    /* Uint16 bfReserved1; */
+    /* Uint16 bfReserved2; */
+    Uint32 bfOffBits;
 
     /* The Win32 BITMAPINFOHEADER struct (40 bytes) */
-    Uint32 biSize = 0;
+    Uint32 biSize;
     Sint32 biWidth = 0;
     Sint32 biHeight = 0;
-    /* Uint16 biPlanes = 0; */
+    /* Uint16 biPlanes; */
     Uint16 biBitCount = 0;
     Uint32 biCompression = 0;
-    /* Uint32 biSizeImage = 0; */
-    /* Sint32 biXPelsPerMeter = 0; */
-    /* Sint32 biYPelsPerMeter = 0; */
+    /* Uint32 biSizeImage; */
+    /* Sint32 biXPelsPerMeter; */
+    /* Sint32 biYPelsPerMeter; */
     Uint32 biClrUsed = 0;
-    /* Uint32 biClrImportant = 0; */
-
-    (void) haveRGBMasks;
-    (void) haveAlphaMask;
+    /* Uint32 biClrImportant; */
 
     /* Make sure we are passed a valid data source */
     surface = NULL;
@@ -148,10 +230,10 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
         was_error = SDL_TRUE;
         goto done;
     }
-    /* bfSize = */ SDL_ReadLE32(src);
+    /* bfSize      = */ SDL_ReadLE32(src);
     /* bfReserved1 = */ SDL_ReadLE16(src);
     /* bfReserved2 = */ SDL_ReadLE16(src);
-    bfOffBits = SDL_ReadLE32(src);
+    bfOffBits   = SDL_ReadLE32(src);
 
     /* Read the Win32 BITMAPINFOHEADER */
     biSize = SDL_ReadLE32(src);
@@ -161,6 +243,11 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
         /* biPlanes = */ SDL_ReadLE16(src);
         biBitCount = SDL_ReadLE16(src);
         biCompression = BI_RGB;
+        /* biSizeImage = 0; */
+        /* biXPelsPerMeter = 0; */
+        /* biYPelsPerMeter = 0; */
+        biClrUsed = 0;
+        /* biClrImportant = 0; */
     } else if (biSize >= 40) {  /* some version of BITMAPINFOHEADER */
         Uint32 headerSize;
         biWidth = SDL_ReadLE32(src);
@@ -175,15 +262,7 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
         /* biClrImportant = */ SDL_ReadLE32(src);
 
         /* 64 == BITMAPCOREHEADER2, an incompatible OS/2 2.x extension. Skip this stuff for now. */
-        if (biSize == 64) {
-            /* ignore these extra fields. */
-            if (biCompression == BI_BITFIELDS) {
-                /* this value is actually huffman compression in this variant. */
-                SDL_SetError("Compressed BMP files not supported");
-                was_error = SDL_TRUE;
-                goto done;
-            }
-        } else {
+        if (biSize != 64) {
             /* This is complicated. If compression is BI_BITFIELDS, then
                we have 3 DWORDS that specify the RGB masks. This is either
                stored here in an BITMAPV2INFOHEADER (which only differs in
@@ -226,6 +305,11 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
             SDL_RWseek(src, (biSize - headerSize), RW_SEEK_CUR);
         }
     }
+    if (biWidth <= 0 || biHeight == 0) {
+        SDL_SetError("BMP file with bad dimensions (%dx%d)", biWidth, biHeight);
+        was_error = SDL_TRUE;
+        goto done;
+    }
     if (biHeight < 0) {
         topDown = SDL_TRUE;
         biHeight = -biHeight;
@@ -246,12 +330,13 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
         ExpandBMP = biBitCount;
         biBitCount = 8;
         break;
+    case 0:
     case 2:
     case 3:
     case 5:
     case 6:
     case 7:
-		SDL_SetError("%d-bpp BMP images are not supported", biBitCount);
+        SDL_SetError("%d-bpp BMP images are not supported", biBitCount);
         was_error = SDL_TRUE;
         goto done;
     default:
@@ -259,7 +344,7 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
         break;
     }
 
-    /* We don't support any BMP compression right now */
+    /* RLE4 and RLE8 BMP compression is supported */
     switch (biCompression) {
     case BI_RGB:
         /* If there are no masks, use the defaults */
@@ -301,9 +386,7 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
         break;  /* we handled this in the info header. */
 
     default:
-        SDL_SetError("Compressed BMP files not supported");
-        was_error = SDL_TRUE;
-        goto done;
+        break;
     }
 
     /* Create a compatible surface, note that the colors are RGB ordered */
@@ -318,31 +401,19 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
     /* Load the palette, if any */
     palette = (surface->format)->palette;
     if (palette) {
-        SDL_assert(biBitCount <= 8);
-        if (biClrUsed == 0) {
-            biClrUsed = 1 << biBitCount;
-		} else if (biClrUsed > (Uint32)(1 << biBitCount)) {
-			SDL_SetError("BMP file has an invalid number of colors");
-			was_error = SDL_TRUE;
-			goto done;
-        }
-        if ((int) biClrUsed > palette->ncolors) {
-            SDL_Color *colors;
-            int ncolors = biClrUsed;
-            colors =
-                (SDL_Color *) SDL_realloc(palette->colors,
-                                          ncolors *
-                                          sizeof(*palette->colors));
-            if (!colors) {
-                SDL_OutOfMemory();
-                was_error = SDL_TRUE;
-                goto done;
-            }
-            palette->ncolors = ncolors;
-            palette->colors = colors;
-        } else if ((int) biClrUsed < palette->ncolors) {
-            palette->ncolors = biClrUsed;
+        if (SDL_RWseek(src, fp_offset+14+biSize, RW_SEEK_SET) < 0) {
+            SDL_Error(SDL_EFSEEK);
+            was_error = SDL_TRUE;
+            goto done;
         }
+
+        /*
+        | guich: always use 1<<bpp b/c some bitmaps can bring wrong information
+        | for colorsUsed
+        */
+        /* if (biClrUsed == 0) {  */
+        biClrUsed = 1 << biBitCount;
+        /* } */
         if (biSize == 12) {
             for (i = 0; i < (int) biClrUsed; ++i) {
                 SDL_RWread(src, &palette->colors[i].b, 1, 1);
@@ -364,6 +435,7 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
                 palette->colors[i].a = SDL_ALPHA_OPAQUE;
             }
         }
+        palette->ncolors = biClrUsed;
     }
 
     /* Read the surface pixels.  Note that the bmp image is upside down */
@@ -372,6 +444,11 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
         was_error = SDL_TRUE;
         goto done;
     }
+    if ((biCompression == BI_RLE4) || (biCompression == BI_RLE8)) {
+        was_error = (SDL_bool)readRlePixels(surface, src, biCompression == BI_RLE8);
+        if (was_error) SDL_SetError("Error reading from BMP");
+        goto done;
+    }
     top = (Uint8 *)surface->pixels;
     end = (Uint8 *)surface->pixels+(surface->h*surface->pitch);
     switch (ExpandBMP) {
@@ -407,11 +484,11 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
                         }
                     }
                     bits[i] = (pixel >> shift);
-					if (bits[i] >= biClrUsed) {
-						SDL_SetError("A BMP image contains a pixel with a color out of the palette");
-						was_error = SDL_TRUE;
-						goto done;
-					}
+                    if (bits[i] >= biClrUsed) {
+                        SDL_SetError("A BMP image contains a pixel with a color out of the palette");
+                        was_error = SDL_TRUE;
+                        goto done;
+                    }
                     pixel <<= ExpandBMP;
                 }
             }
@@ -423,15 +500,15 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
                 was_error = SDL_TRUE;
                 goto done;
             }
-			if (biBitCount == 8 && palette && biClrUsed < (Uint32)(1 << biBitCount)) {
-				for (i = 0; i < surface->w; ++i) {
-					if (bits[i] >= biClrUsed) {
-						SDL_SetError("A BMP image contains a pixel with a color out of the palette");
-						was_error = SDL_TRUE;
-						goto done;
-					}
-				}
-			}
+            if (biBitCount == 8 && palette && biClrUsed < (1u << biBitCount)) {
+                for (i = 0; i < surface->w; ++i) {
+                    if (bits[i] >= biClrUsed) {
+                        SDL_SetError("A BMP image contains a pixel with a color out of the palette");
+                        was_error = SDL_TRUE;
+                        goto done;
+                    }
+                }
+            }
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
             /* Byte-swap the pixels if needed. Note that the 24bpp
                case has already been taken care of above. */
@@ -475,7 +552,9 @@ SDL_LoadBMP_RW(SDL_RWops * src, int freesrc)
         if (src) {
             SDL_RWseek(src, fp_offset, RW_SEEK_SET);
         }
-        SDL_FreeSurface(surface);
+        if (surface) {
+            SDL_FreeSurface(surface);
+        }
         surface = NULL;
     }
     if (freesrc && src) {
