@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,10 +18,16 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
+
+#ifndef SDL_nullevents_c_h_
+#define SDL_nullevents_c_h_
+
 #include "../../SDL_internal.h"
 
 #include "SDL_nullvideo.h"
 
 extern void DUMMY_PumpEvents(_THIS);
 
+#endif /* SDL_nullevents_c_h_ */
+
 /* vi: set ts=4 sw=4 expandtab: */
