@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,13 +20,14 @@
 */
 #include "../../SDL_internal.h"
 
-#ifndef _SDL_x11clipboard_h
-#define _SDL_x11clipboard_h
+#ifndef SDL_x11clipboard_h_
+#define SDL_x11clipboard_h_
 
 extern int X11_SetClipboardText(_THIS, const char *text);
 extern char *X11_GetClipboardText(_THIS);
 extern SDL_bool X11_HasClipboardText(_THIS);
+extern Atom X11_GetSDLCutBufferClipboardType(Display *display);
 
-#endif /* _SDL_x11clipboard_h */
+#endif /* SDL_x11clipboard_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
