@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -39,7 +39,6 @@
 #include "SDL_hints.h"
 #include "SDL_timer.h"
 #include "SDL_syswm.h"
-#include "SDL_assert.h"
 
 #include <stdio.h>
 
@@ -433,8 +432,12 @@ X11_DispatchFocusOut(_THIS, SDL_WindowData *data)
 static void
 X11_DispatchMapNotify(SDL_WindowData *data)
 {
-    SDL_SendWindowEvent(data->window, SDL_WINDOWEVENT_SHOWN, 0, 0);
-    SDL_SendWindowEvent(data->window, SDL_WINDOWEVENT_RESTORED, 0, 0);
+    SDL_Window *window = data->window;
+    SDL_SendWindowEvent(window, SDL_WINDOWEVENT_RESTORED, 0, 0);
+    SDL_SendWindowEvent(window, SDL_WINDOWEVENT_SHOWN, 0, 0);
+    if (!(window->flags & SDL_WINDOW_HIDDEN) && (window->flags & SDL_WINDOW_INPUT_FOCUS)) {
+        SDL_UpdateWindowGrab(window);
+    }
 }
 
 static void
@@ -642,6 +645,29 @@ X11_HandleClipboardEvent(_THIS, const XEvent *xevent)
     }
 }
 
+static Bool
+isMapNotify(Display *display, XEvent *ev, XPointer arg)
+{
+    XUnmapEvent *unmap;
+
+    unmap = (XUnmapEvent*) arg;
+
+    return ev->type == MapNotify &&
+        ev->xmap.window == unmap->window &&
+        ev->xmap.serial == unmap->serial;
+}
+
+static Bool
+isReparentNotify(Display *display, XEvent *ev, XPointer arg)
+{
+    XUnmapEvent *unmap;
+
+    unmap = (XUnmapEvent*) arg;
+
+    return ev->type == ReparentNotify &&
+        ev->xreparent.window == unmap->window &&
+        ev->xreparent.serial == unmap->serial;
+}
 
 static void
 X11_DispatchEvent(_THIS)
@@ -683,7 +709,7 @@ X11_DispatchEvent(_THIS)
         /* But only if we're using one of the DBus IMEs, otherwise
            some XIM IMEs will generate duplicate events */
         if (orig_keycode) {
-#if defined(HAVE_IBUS_IBUS_H) || defined(HAVE_FCITX_FRONTEND_H)
+#if defined(HAVE_IBUS_IBUS_H) || defined(HAVE_FCITX)
             SDL_Scancode scancode = videodata->key_layout[orig_keycode];
             videodata->filter_code = orig_keycode;
             videodata->filter_time = xevent.xkey.time;
@@ -818,9 +844,9 @@ X11_DispatchEvent(_THIS)
                 break;
             }
 
-            if (xevent.xfocus.detail == NotifyInferior) {
+            if (xevent.xfocus.detail == NotifyInferior || xevent.xfocus.detail == NotifyPointer) {
 #ifdef DEBUG_XEVENTS
-                printf("window %p: FocusIn (NotifierInferior, ignoring)\n", data);
+                printf("window %p: FocusIn (NotifyInferior/NotifyPointer, ignoring)\n", data);
 #endif
                 break;
             }
@@ -851,10 +877,12 @@ X11_DispatchEvent(_THIS)
 #endif
                 break;
             }
-            if (xevent.xfocus.detail == NotifyInferior) {
-                /* We still have focus if a child gets focus */
+            if (xevent.xfocus.detail == NotifyInferior || xevent.xfocus.detail == NotifyPointer) {
+                /* We still have focus if a child gets focus. We also don't
+                   care about the position of the pointer when the keyboard
+                   focus changed. */
 #ifdef DEBUG_XEVENTS
-                printf("window %p: FocusOut (NotifierInferior, ignoring)\n", data);
+                printf("window %p: FocusOut (NotifyInferior/NotifyPointer, ignoring)\n", data);
 #endif
                 break;
             }
@@ -898,7 +926,7 @@ X11_DispatchEvent(_THIS)
             }
 #endif
             /* */
-            SDL_zero(text);
+            SDL_zeroa(text);
 #ifdef X_HAVE_UTF8_STRING
             if (data->ic) {
                 X11_Xutf8LookupString(data->ic, &xevent.xkey, text, sizeof(text),
@@ -946,10 +974,17 @@ X11_DispatchEvent(_THIS)
 
         /* Have we been iconified? */
     case UnmapNotify:{
+            XEvent ev;
+
 #ifdef DEBUG_XEVENTS
             printf("window %p: UnmapNotify!\n", data);
 #endif
-            X11_DispatchUnmapNotify(data);
+
+            if (X11_XCheckIfEvent(display, &ev, &isReparentNotify, (XPointer)&xevent.xunmap)) {
+                X11_XCheckIfEvent(display, &ev, &isMapNotify, (XPointer)&xevent.xunmap);
+            } else {
+                X11_DispatchUnmapNotify(data);
+            }
         }
         break;
 
@@ -1346,19 +1381,22 @@ X11_DispatchEvent(_THIS)
                 X11_ReadProperty(&p, display, data->xwindow, videodata->PRIMARY);
 
                 if (p.format == 8) {
-                    /* !!! FIXME: don't use strtok here. It's not reentrant and not in SDL_stdinc. */
-                    char* name = X11_XGetAtomName(display, target);
-                    char *token = strtok((char *) p.data, "\r\n");
-                    while (token != NULL) {
-                        if (SDL_strcmp("text/plain", name)==0) {
-                            SDL_SendDropText(data->window, token);
-                        } else if (SDL_strcmp("text/uri-list", name)==0) {
-                            char *fn = X11_URIToLocal(token);
-                            if (fn) {
-                                SDL_SendDropFile(data->window, fn);
+                    char *saveptr = NULL;
+                    char *name = X11_XGetAtomName(display, target);
+                    if (name) {
+                        char *token = SDL_strtokr((char *) p.data, "\r\n", &saveptr);
+                        while (token != NULL) {
+                            if (SDL_strcmp("text/plain", name) == 0) {
+                                SDL_SendDropText(data->window, token);
+                            } else if (SDL_strcmp("text/uri-list", name) == 0) {
+                                char *fn = X11_URIToLocal(token);
+                                if (fn) {
+                                    SDL_SendDropFile(data->window, fn);
+                                }
                             }
+                            token = SDL_strtokr(NULL, "\r\n", &saveptr);
                         }
-                        token = strtok(NULL, "\r\n");
+                        X11_XFree(name);
                     }
                     SDL_SendDropComplete(data->window);
                 }
